"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const process = require("process");
const chalk_1 = require("chalk");
const cli_1 = require("../../cli");
const session_1 = require("../../session");
const models_1 = require("../../models");
exports.command = 'install-assistant';
exports.desc = 'Install an assistant';
exports.builder = {
    verbose: {
        demandOption: false,
        describe: 'Include remote logs',
        type: 'boolean',
        alias: 'v',
    },
    path: {
        demandOption: false,
        describe: 'Path to local assistant folder, can be a zip',
        type: 'string',
        conflicts: 'id',
        alias: 'p',
    },
    id: {
        demandOption: false,
        describe: 'Your assistant id from the console, format: proj_XXXX',
        type: 'string',
        conflicts: 'path',
        alias: 'i',
    },
};
exports.handler = (argv) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const verbose = JSON.parse(JSON.stringify(argv)).verbose;
    const assistantPath = JSON.parse(JSON.stringify(argv)).path;
    const id = JSON.parse(JSON.stringify(argv)).id;
    let assistantId;
    if (assistantPath === undefined) {
        if (cli_1.default.config.user === undefined) {
            cli_1.default.stream.error(`No user information to connect to the console, you must login first with ${chalk_1.default.blue('sam login')}`);
            process.exit(0);
            return;
        }
        const client = new session_1.ConsoleClient();
        if (id !== undefined && id.length > 0) {
            assistantId = id;
        }
        else {
            try {
                cli_1.default.stream.loading('Fetching assistants');
                assistantId = yield client.assistantList(cli_1.default.config.user.id)
                    .then((assistants) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    cli_1.default.stream.done();
                    const assistantNames = assistants.map(assistant => assistant.title);
                    if (assistantNames.length === 0)
                        throw new Error('No assistants found on your account.');
                    if (assistantNames.length === 1) {
                        cli_1.default.stream.println(`Found 1 assistant named ${assistants[0].title}`);
                        return assistants[0].id;
                    }
                    return yield cli_1.default.prompt.promptList('assistant', 'Choose the assistant you wish to install on the device', assistantNames)
                        .then(response => {
                        const assistantSelected = assistants.find(value => value.title === response['assistant']);
                        if (assistantSelected === undefined)
                            throw new Error('No assistant choosen');
                        return assistantSelected.id;
                    });
                }));
            }
            catch (e) {
                cli_1.default.stream.error(`Error fetching the assistant, reason: ${e.message}`);
                process.exit(0);
                return;
            }
        }
    }
    try {
        const client = new session_1.PegasusClient(cli_1.default.config.pegasusConfiguration);
        let taskId;
        if (assistantPath !== undefined && assistantPath.length > 0) {
            cli_1.default.stream.loading('Sending assistant to device');
            taskId = yield client.put_assistant_zip(assistantPath, true, false, undefined);
            cli_1.default.stream.success();
        }
        else if (assistantId !== undefined && assistantId.length > 0 && cli_1.default.config.consoleToken !== undefined) {
            taskId = yield client.put_assistant_from_console(assistantId, cli_1.default.config.consoleToken.token, false, true, undefined);
        }
        if (taskId) {
            const task = yield client.get_tasks_logs(taskId, (task, logs) => {
                if (task.status === models_1.TaskStatus.Queued) {
                    cli_1.default.stream.println('Task is queued, waiting');
                }
                if (verbose) {
                    logs.forEach(log => {
                        if (log.source === models_1.TaskLogSource.Info) {
                            cli_1.default.stream.println(log.line);
                        }
                        else {
                            cli_1.default.stream.warning(log.line);
                        }
                    });
                }
            });
            if (task.status === models_1.TaskStatus.Failed) {
                cli_1.default.stream.error('Assistant installation failed');
            }
            else {
                cli_1.default.stream.success('Assistant installed');
            }
        }
    }
    catch (e) {
        cli_1.default.stream.error(`Failed to install platform: ${e.stack}`);
        process.exit();
    }
});
//# sourceMappingURL=install-assistant.js.map